#ifndef CMDITEM_H
/*
   Definitions for command items (commands and command parameters) 

   Copyright  1992-1998 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16, all ri

   $Log: cmditem.h,v $
   Revision 1.2  2008/02/05 11:36:06  kraemer
   proper exit/quit handling

   Revision 1.1.1.1  1999/02/02 20:13:14  kraemer
    


*/

#include "exec/nodes.h"
#include "exec/lists.h"
#include "cmderr.h"
                                      /* */
union UCMDITEM { double dd; float rr; long ll; short ii; };
                                      /* structure for commands + parameters */
struct SCMDITEM {
   struct Node ln;
   int nAbbr;                         /* abbreviation length */
   long lType;                        /* item data type */
   long lFlag;                        /* item flags */
   union UCMDITEM uuDef;              /* default values (non-char) */
   union UCMDITEM uuVal;              /* actual  values (non-char) */
   char *pcDef;                       /* default values (char) */
   char *pcVal;                       /* actual  values (char) */
   void *pvVal;                       /* ptr. to actual value field */
   char *pcCom;                       /* comment text */
   char *pcAlias;                     /* name of alias */
   struct SCMDITEM *psCIAlias;        /* ptr. to alias */
   struct List lh;                    /* list header for command parameters */
   int   (*eFct)();                   /* function entry point */
   int  nParReq;                      /* total # of required parameters */
   int  nParPos;                      /* total # of positional parameters */
   int  nParOpt;                      /* total # of optional parameters */
   int  nPar;                         /* total # of parameters */
};
                                      /* parameter item types */
enum {
   CMDITEM_BOOL=0, CMDITEM_SHORT, CMDITEM_LONG,
   CMDITEM_FLOAT, CMDITEM_DOUBLE,
   CMDITEM_CHARSTR, CMDITEM_FILENAME,
};
                                      /* command item types */
enum {
   CMDITEM_CMDANY=0, 
   CMDITEM_CMDQUIT, CMDITEM_CMDEXIT, CMDITEM_CMDEXEC,
};
                                      /* item flags */
#define CMDITEM_REPLACEDEFAULT 0x00000001  /* replaceable default */
#define CMDITEM_MASKDEF (CMDITEM_REPLACEDEFAULT)
#define CMDITEM_HIDE           0x00000002  /* a hidden item */
#define CMDITEM_ISCMD          0x00000004  /* item is a command */
#define CMDITEM_REQPAR         0x00000010
#define CMDITEM_POSPAR         0x00000020
#define CMDITEM_OPTPAR         0x00000040
#define CMDITEM_MASKPOS (CMDITEM_REQPAR|CMDITEM_POSPAR|CMDITEM_OPTPAR)
#if defined(CMDFLAG_OFF)
#define CMDITEM_EXIT           0x00000100  /* this is the exit command */
#define CMDITEM_QUIT           0x00000200  /* this is the quit command */
#define CMDITEM_EXEC           0x00000400  /* this is the exec command */
#endif
                                      /* */
                                      /* global flags */
#define CMDITEM_DEBUG  0x80000000     /* general check flag */
#define CMDITEM_QUIET  0x10000000     /* suppress error messages */
#define CMDITEM_NOMSG  CMD_QUIET

/*
   --- function prototypes
*/

int _cmditemDefault2Value( struct SCMDITEM *, unsigned long );
int _cmditemDup    ( struct SCMDITEM *, struct SCMDITEM *, unsigned long );
int _cmditemError  ( struct SCMDERR  *, unsigned long );
int _cmditemFree   ( struct SCMDITEM *, unsigned long ); 
int _cmditemFromStr( const char *, int, struct SCMDITEM *, unsigned long );
int _cmditemInit   ( struct SCMDITEM *, unsigned long ); 
int _cmditemInspect( struct SCMDITEM *, unsigned long ); 
int _cmditemValue2Default( struct SCMDITEM *, unsigned long );
int _cmditemValueFromStr( const char *, int, struct SCMDITEM *, unsigned long );

int _cmditemListDefault2Value( struct List *, unsigned long );
int _cmditemListDup       ( struct List *, struct List *, unsigned long );
int _cmditemListFree      ( struct List *, unsigned long );
int _cmditemListInspect   ( struct List *, unsigned long );
struct SCMDITEM *_cmditemListFindName( struct List *, const char *, int, unsigned long );
int _cmditemListNodeAdd   ( struct List *, struct SCMDITEM *, unsigned long );
int _cmditemListNodeAbbrev( struct List *, struct SCMDITEM *, unsigned long );
int _cmditemListResolveAlias( struct List *, unsigned long );
int _cmditemListValue2Default( struct List *, unsigned long );

#if defined(LATER)

   _cmditemList?( struct List *, const char *, int, unsigned long );
      o list command(s)
      o from old cmdlist() 

   _cmditem?( pc, nc, ... ) to process <name>...(<arg>) ??

#endif

/*
   --- error codes
*/
enum { ERRCMDITEM_SYS=1,
       ERRCMDITEM_BADADDR, ERRCMDITEM_SYNTAX, ERRCMDITEM_CONV,
       ERRCMDITEM_BADTYPE, ERRCMDITEM_BALANCE,
       ERRCMDITEM_AMBIGUOUS, ERRCMDITEM_NOTFOUND,
       ERRCMDITEM_SELFALIAS, ERRCMDITEM_NESTALIAS,
       ERRCMDITEM_BADNODE,
};

#define CMDITEM_H
#endif

