#ifndef TRXLMD_H
#define TRXLMD_H
/*
   
   Definitions for listmode data I/O
   =================================

   General
   -------
   
   o for plain native:
     o some format specifier/eye catcher/version index,
       e.g. a la IFF 
     o byte swap ?

   Copyright  1992-2002 M.Kraemer, GSI Darmstadt 

   $Log: trxlmd.h,v $
   Revision 1.2  2002/11/25 16:28:46  kraemer
   volume mode

   Revision 1.1  2001/01/09 14:54:32  kraemer
   *** empty log message ***


*/

#include "m.h"
#include "cmderr.h"
#include "trxparticle.h"
#include "trxeval.h"

                                      /* masks */
#define TRXLMD_MASKGLOBAL 0xFF000000  /* global (e.g. debug) */
#define TRXLMD_MASKFUNC   0x00FF0000  /* function specific flags */
#define TRXLMD_MASKQUAL   0x0000FFFF  /* for additional qualifiers */

                                      /* flags */
#define TRXLMD_DEBUG      0x80000000  /* general check flag */
#define TRXLMD_CHECK      TRXLMD_DEBUG/* general check flag */
#define TRXLMD_QUIET      0x10000000  /* suppress error messages */
#define TRXLMD_NOMSG      TRXLMD_QUIET


                                      /* here's the main structure */
struct STRXLMD {
   unsigned long ulFlag;              /* flag bits */
   unsigned long ulMode;              /* read/write/append */
   int iFmt;                          /* output format */
   int iDevice;                       /*  */
   int iFH;                           /* file handle */ 
   char *pcFile;                      /* file name */
   struct STRXEVAL sEval;             /* for evaluation on-the-fly (deferred
   mode) */
   struct SCMDERR sCE;
};
                                      /* flag bits */
#define TRXLMD_VOLMODE 0x00000001				      
                                      /* */
enum { TRXLMD_FMTNATIVE=0, TRXLMD_FMTMBS };
                                      /* */
enum { TRXLMD_DEVDISK=0, TRXLMD_DEVTAPE, TRXLMD_DEVEVAL };
                                      /* */
enum { TRXLMD_EOF=1 };
                                      /* */
enum { TRXLMD_READ=0, TRXLMD_WRITE };
                                      /* */
/*

   Functions for listmode handling
   -------------------------------
   
   TRXLmdImportEval  ( lmd, eval, flag )
                     o initialize deferred mode
                   8 x to test
                     o calls
                       o TRXEvalInit()  
                       o TRXLmdFree()
                   
   TRXLmdClose       ( lmd, flag )
                     o close listmode file
                   3 x to test
		     x deferred mode (to test)
                     o calls
                   
   TRXLmdFree        ( lmd, flag )
                     o release resources
                   4 x to test
                     o calls
                       o TRXLmdInit()  
                       o TRXLmdClose()  
                       o TRXEvalFree()  
                   
   TRXLmdInit        ( lmd, flag )
                     o initialize
                   2 x to test
                     o calls
                       o TRXEvalInit()  
                   
   TRXLmdInspect     ( lmd, flag )
                     o inspect
                   1 x to test
                     o calls
                       o TRXEvalInspect()  
                   
   TRXLmdOpen        ( lmd, file, options, flag )
                     o open a listmode file
                   5 x to test
                     x volmode option
		     o buffered I/O
                     o options
                       o "r", "w", "v"
                       o format(mbs), format(native)
                     ? set ulMode   
                     o support various media
                       o disk / mmap()
                       o tape / _MT*()  
                     o account for direct evaluations  
                     o calls
                       o TRXLmdFree()  
                       o TRXLmdClose()  
                   
   TRXLmdRead        ( lmd, evt, flag )
                     o reads an event from buffer/listmode file
                   7 x to test
                     ! consider bytesex on read
                     o returns
                       o rc = CMDERR_MASK: I/O error
                       o rc = TRXLMD_EOF : EOF reached
                       o rc = 0          : OK 
		     o buffered input
                     o support various media
                       o disk / mmap()
                       o tape / _MT*()  
                       o evaluation routines
                     o support various formats
                       o plain native
                       o MBS ( $IGOOWRITE() )     
                     o calls
                   
   TRXLmdRewind      ( lmd, flag )
                     o reposition at file start
                   9 x to test
                     o calls
                   
   TRXLmdWrite       ( lmd, evt, flag )
                     o write an event to buffer/listmode file
                   6 x to test
		     o buffered output
                     o support various media
                       o disk / mmap()
                       o tape / _MT*()  
                       o deferred mode: evaluation routines
                         o on EOR: trigger EOF evaluation
                     o support various formats
                       o plain native
                       o MBS ( $IGOOWRITE() )     
                     o calls
                   

*/                                      
                                      /* function prototypes */
int TRXLmdImportEval( struct STRXLMD *, struct STRXEVAL *, unsigned long );
int TRXLmdClose     ( struct STRXLMD *, unsigned long );
int TRXLmdFree      ( struct STRXLMD *, unsigned long );
int TRXLmdInit      ( struct STRXLMD *, unsigned long );
int TRXLmdInspect   ( struct STRXLMD *, unsigned long );
int TRXLmdOpen      ( struct STRXLMD *, const char *, const char *, unsigned long );
int TRXLmdRead      ( struct STRXLMD *, void *, unsigned long );
int TRXLmdRewind    ( struct STRXLMD *, unsigned long );
int TRXLmdWrite     ( struct STRXLMD *, void *, unsigned long );

struct STRXLMD *_TRXLmdTest( unsigned long );
                                      /* error handling */
int TRXLmdError( struct SCMDERR *, unsigned long );

enum { ERRTRXLMD_SYS=1,
       ERRTRXLMD_BADADDR,
       ERRTRXLMD_BADEVAL,
       ERRTRXLMD_BADFILE,
       ERRTRXLMD_BADOPTION,
       ERRTRXLMD_NOFILEOPEN,
       ERRTRXLMD_BADFILEOP,
       ERRTRXLMD_EVALFAILED,
};
#endif
